@@ -8,7 +8,7 @@
     stop("This package requires R 3.5.0 or later")
   if(interactive()) {
     packageStartupMessage(blue(paste("[]==================================================================[]")),appendLF=TRUE)
-    packageStartupMessage(blue(paste("[] Linear Mixed Equations 4 Breeding (lme4breeding) 1.0.5 (2025-01) []",sep="")),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[] Linear Mixed Equations 4 Breeding (lme4breeding) 1.0.6 (2025-05) []",sep="")),appendLF=TRUE)
     packageStartupMessage(paste0(blue("[] Author: Giovanny Covarrubias-Pazaran",paste0(bgGreen(white(" ")), bgWhite(magenta("*")), bgRed(white(" "))),"                        []")),appendLF=TRUE)
     packageStartupMessage(blue("[] Dedicated to the University of Chapingo and UW-Madison           []"),appendLF=TRUE)
     packageStartupMessage(blue("[] Type 'vignette('lmebreed.gxe')' for a short tutorial             []"),appendLF=TRUE)
@@ -71,11 +71,6 @@ umat <- function(formula, relmat, data, addmat){
     }
     ZrZrt[[iProv]] <- Zr %*% t(Zr)
   }
-  # part0 <- Reduce("+",ZrZrt)
-  # part0[which(part0 > 0)]=1
-  # part0 <- as(rotate(part0), Class = "dgCMatrix")
-  # dim(Zr)
-  # Matrix::image(part0)
   # build the U nxn matrix
   Ul <- Dl <- Zu <- list()
   for(iProv in idProvided){
@@ -147,7 +142,7 @@ leg <- function(x,n=1,u=-1,v=1, intercept=TRUE, intercept1=FALSE){
 }
 
 ####
-getMME <- function(object, vc=NULL, recordsToKeep=NULL){
+getMME <- function(object, vc=NULL, recordsToUse=NULL){
   if(is.null(vc)){
     vc <- VarCorr(object) #object %>% VarCorr %>% as_tibble # extract estimated variance components (vc)
   }
@@ -162,11 +157,11 @@ getMME <- function(object, vc=NULL, recordsToKeep=NULL){
   Z <- getME(object, "Z") #%>% as.matrix # Design matrix random effects
   y <- getME(object, "y") #%>% as.matrix # Design matrix random effects
   
-  if(!is.null(recordsToKeep)){
-    X <- X[recordsToKeep,, drop=FALSE]
-    Z <- Z[recordsToKeep,, drop=FALSE]
-    y <- y[recordsToKeep]
-    Ri <- Ri[recordsToKeep,recordsToKeep]
+  if(!is.null(recordsToUse)){
+    X <- X[recordsToUse,, drop=FALSE]
+    Z <- Z[recordsToUse,, drop=FALSE]
+    y <- y[recordsToUse]
+    Ri <- Ri[recordsToUse,recordsToUse]
   }
   # Mixed Model Equation (HENDERSON 1986; SEARLE et al. 2006)
   C11 <- t(X) %*% Ri %*% X
@@ -243,9 +238,9 @@ getMME <- function(object, vc=NULL, recordsToKeep=NULL){
     C_invROT <- t(ROT) %*% C_inv %*% (ROT)
     rownames(buROT) <- rn
     colnames(C_invROT) <- rownames(C_invROT) <- rn
-    return(list(Ci=C_invROT, bu=buROT))
+    return(list(Ci=C_invROT, bu=buROT, RHS=RHS))
   }else{
-    return(list(Ci=C_inv, bu=bu))
+    return(list(Ci=C_inv, bu=bu, RHS=RHS))
   }
   
   
@@ -253,26 +248,19 @@ getMME <- function(object, vc=NULL, recordsToKeep=NULL){
 
 stackTrait <- function(data, traits){
   
-  '%!in%' <- function(x,y)!('%in%'(x,y)) 
   dataScaled <- data
-  # remove traits not present in the dataset
   traits <- intersect(traits, colnames(data) )
-  # check that traits are numeric
-  columnTypesTraits <- unlist(lapply(data[traits],class)) 
-  badones <- which(columnTypesTraits %!in% c("integer","numeric") )
-  if(length(badones) > 0){stop("some of your selected traits are not numeric. Please correct the traits provided.", call. = FALSE)}
-  # identify possible idvars
   idvars <- setdiff(colnames(data), traits)
   for(iTrait in traits){
     dataScaled[,iTrait] <- scale(dataScaled[,iTrait])
   }
   columnTypes <- unlist(lapply(data[idvars],class)) 
-  columnTypes <- columnTypes[which(columnTypes %in% c("factor","character"))]
+  columnTypes <- columnTypes[which(columnTypes %in% c("factor","character","integer"))]
   idvars <- intersect(idvars,names(columnTypes))
-  data2 <- reshape(data[,c(idvars, traits)], idvar = idvars, varying = traits,
+  data2 <- reshape(data, idvar = idvars, varying = traits,
                    timevar = "trait",
                    times = traits,v.names = "value", direction = "long")
-  data2Scaled <- reshape(dataScaled[,c(idvars, traits)], idvar = idvars, varying = traits,
+  data2Scaled <- reshape(dataScaled, idvar = idvars, varying = traits,
                          timevar = "trait",
                          times = traits,v.names = "value", direction = "long")
   data2 <- as.data.frame(data2)
@@ -571,7 +559,8 @@ smm <- function(x){
   }else{
     if(!is.character(x) & !is.factor(x)){
       namess <- as.character(substitute(list(x)))[-1L]
-      dummy <- as(Matrix(x,ncol=1), Class = "dgCMatrix"); colnames(dummy) <- namess
+      dummy <- as(as(as( Matrix(x,ncol=1),  "dMatrix"), "generalMatrix"), "CsparseMatrix") 
+      colnames(dummy) <- namess
       mm <- diag(ncol(dummy));
     }else{
       dummy <- x
@@ -962,7 +951,7 @@ atcg1234 <- function(data, ploidy=2, format="ATCG", maf=0, multi=TRUE, silent=FA
   return(list(M=M,ref.alleles=tmp))
 }
 
-build.HMM <- function(M1,M2, custom.hyb=NULL, return.combos.only=FALSE, separator=":"){
+build.HMM <- function(M1,M2, custom.hyb=NULL, return.combos.only=FALSE, separator=":", n.batch=1000, verbose=TRUE){
   # build hybrid marker matrix
   
   if(!is.null(custom.hyb)){
@@ -990,71 +979,91 @@ build.HMM <- function(M1,M2, custom.hyb=NULL, return.combos.only=FALSE, separato
     
     if(all(checkM1 == c(1,1,0))){ # homo markers were coded correctly as -1,1
     }else if(all(checkM1 == c(0,1,0)) | all(checkM1 == c(1,0,0))){ # homo markers were coded as 0 1
-      warning("Either -1 or 1 alleles not detected in M1, we assume you have coded homozygotes \n       as 0 and 1 instead of -1 and 1. We'll fix it.\n")
+      message("Either -1 or 1 alleles not detected in M1, we assume you have coded homozygotes \n       as 0 and 1 instead of -1 and 1. We'll fix it.\n")
     }else if(all(checkM1 == c(0,0,1))){ # homo markers were coded as 0 2
-      warning("Either -1 or 1 alleles not detected in M1, we assume you have coded homozygotes \n       as 0 and 2 instead of -1 and 1. We'll fix it.\n")
+      message("Either -1 or 1 alleles not detected in M1, we assume you have coded homozygotes \n       as 0 and 2 instead of -1 and 1. We'll fix it.\n")
     }
     
     if(all(checkM2 == c(1,1,0))){ # homo markers were coded correctly as -1,1
+      
     }else if(all(checkM2 == c(0,1,0)) | all(checkM2 == c(1,0,0))){ # homo markers were coded as 0 1
-      warning("Either -1 or 1 alleles not detected in M2, we assume you have coded homozygotes \n       as 0 and 1 instead of -1 and 1. We'll fix it.\n")
+      message("Either -1 or 1 alleles not detected in M2, we assume you have coded homozygotes \n       as 0 and 1 instead of -1 and 1. We'll fix it.\n")
     }else if(all(checkM2 == c(0,0,1))){ # homo markers were coded as 0 2
-      warning("Either -1 or 1 alleles not detected in M2, we assume you have coded homozygotes \n       as 0 and 2 instead of -1 and 1. We'll fix it.\n")
+      message("Either -1 or 1 alleles not detected in M2, we assume you have coded homozygotes \n       as 0 and 2 instead of -1 and 1. We'll fix it.\n")
     }
     
-    ## add markers coming from parents M1
-    Z1 <- model.matrix(~Var1-1,pheno);dim(Z1); 
-    colnames(Z1) <- gsub("Var1","",colnames(Z1))
-    M1 <- M1[colnames(Z1),]
-    #M1[1:4,1:4]; Z1[1:4,1:4]; 
-    ## add markers coming from parents M2
-    Z2 <- model.matrix(~Var2-1,pheno);dim(Z2); 
-    colnames(Z2) <- gsub("Var2","",colnames(Z2))
-    M2 <- M2[colnames(Z2),]
-    #M2[1:4,1:4]; Z2[1:4,1:4];  
+    n.batch <- min(c(n.batch,nrow(pheno)))
     
-    ## create the 
-    # Z3 <- model.matrix(~hybrid-1,pheno);dim(Z3);
-    # colnames(Z3) <- gsub("hybrid","",colnames(Z3))
-    # hyb.names <- colnames(Z3)[as.vector(apply(Z3,1,function(x){which(x==1)}))] # names of hybrids
-    hyb.names <- pheno$hybrid
-    ## marker matrix for hybrids one for each parent
-    message(paste("Building hybrid marker matrix for",nrow(Z1),"hybrids\n"))
-    
-    # M1 <- as(M1, Class="dgCMatrix")
-    # M2 <- as(M2, Class="dgCMatrix")
-    # Z1 <- as(Z1, Class="dgCMatrix")
-    # Z2 <- as(Z2, Class="dgCMatrix")
-    
-    message("Extracting M1 contribution\n")
-    if(all(checkM1 == c(1,1,0))){ # homo markers were coded correctly as -1,1
-      Md <- Z1 %*% M1;  # was already converted to -1,1
-    }else if(all(checkM1 == c(0,1,0)) | all(checkM1 == c(1,0,0))){ # homo markers were coded as 0 1
-      Md <- 2*Z1 %*% M1 - 1;  # 2*Z.dent %*% M.dent - 1   # convert to -1,1
-    }else if(all(checkM1 == c(0,0,1))){ # homo markers were coded as 0 2
-      Md <- Z1 %*% M1 - 1;  # Z.dent %*% M.dent - 1   # convert to -1,1
+    if(nrow(pheno)>0){ # if there is hybrids to build
+      ## build the marker matrix for batches of n.batch hybrids
+      batches <- sort(rep(1:1000,min(c(nrow(pheno),n.batch))))
+      pheno$batch <- batches[1:nrow(pheno)]
+      data.usedBatches <- split(pheno, pheno$batch)
+      
+      M1 <- as(M1, "CsparseMatrix")
+      M2 <- as(M2, "CsparseMatrix")
+      # start the loop
+      for(i in 1:length(data.usedBatches)){
+        
+        ## add markers coming from parents M1
+        Z1 <- Matrix::sparse.model.matrix(~Var1-1,data.usedBatches[[i]]);dim(Z1); 
+        colnames(Z1) <- gsub("Var1","",colnames(Z1))
+        M1r <- M1[colnames(Z1),,drop=FALSE]
+        
+        ## add markers coming from parents M2
+        Z2 <- Matrix::sparse.model.matrix(~Var2-1,data.usedBatches[[i]]);dim(Z2); 
+        colnames(Z2) <- gsub("Var2","",colnames(Z2))
+        M2r <- M2[colnames(Z2),, drop=FALSE]
+        
+        
+        hyb.names <- data.usedBatches[[i]]$hybrid
+        ## marker matrix for hybrids one for each parent
+        if(verbose){
+          message(paste("Building hybrid marker matrix for",nrow(Z1),"hybrids\n"))
+          message("Extracting M1 contribution\n")
+        }
+        
+        if(all(checkM1 == c(1,1,0))){ # homo markers were coded correctly as -1,1
+          Md <- Z1 %*% M1r;  # was already converted to -1,1
+        }else if(all(checkM1 == c(0,1,0)) | all(checkM1 == c(1,0,0))){ # homo markers were coded as 0 1
+          Md <- 2*Z1 %*% M1r - 1;  # 2*Z.dent %*% M.dent - 1   # convert to -1,1
+        }else if(all(checkM1 == c(0,0,1))){ # homo markers were coded as 0 2
+          Md <- Z1 %*% M1r - 1;  # Z.dent %*% M.dent - 1   # convert to -1,1
+        }
+        
+        if(verbose){cat("Extracting M2 contribution\n")}
+        if(all(checkM2 == c(1,1,0))){ # homo markers were coded correctly as -1,1
+          Mf <- Z2 %*% M2r;  # was already converted to -1,1
+        }else if(all(checkM2 == c(0,1,0)) | all(checkM2 == c(1,0,0))){ # homo markers were coded as 0 1
+          Mf <- 2*Z2 %*% M2r - 1;  # 2*Z.dent %*% M.dent - 1   # convert to -1,1
+        }else if(all(checkM2 == c(0,0,1))){ # homo markers were coded as 0 2
+          Mf <- Z2 %*% M2r - 1;  # Z.dent %*% M.dent - 1   # convert to -1,1
+        }
+        
+        ## marker matrix coded as additive -1,0,1
+        Mdf <- (Md + Mf)*(1/2) # normal marker matrix for the hybrids
+        rownames(Mdf) <- hyb.names
+        #hist(Mdf)
+        
+        ## dominance matrix for hybrids (0,1 coded)
+        Delta <- 1/2*(1 - Md * Mf) #performs element wise multiplication = Hadamard product
+        rownames(Delta) <- hyb.names
+        
+        if(i == 1){
+          HMM.add <- Mdf
+          HMM.dom <- Delta
+        }else{
+          HMM.add <- rbind(Mdf, HMM.add)
+          HMM.dom <- rbind(Delta, HMM.dom)
+        }
+      }
+    }else{
+      HMM.add <- HMM.dom <- Matrix::Matrix(NA, nrow=0, ncol=ncol(M1)); colnames(M) <- colnames(M1)
     }
     
-    message("Extracting M2 contribution\n")
-    if(all(checkM2 == c(1,1,0))){ # homo markers were coded correctly as -1,1
-      Mf <- Z2 %*% M2;  # was already converted to -1,1
-    }else if(all(checkM2 == c(0,1,0)) | all(checkM2 == c(1,0,0))){ # homo markers were coded as 0 1
-      Mf <- 2*Z2 %*% M2 - 1;  # 2*Z.dent %*% M.dent - 1   # convert to -1,1
-    }else if(all(checkM2 == c(0,0,1))){ # homo markers were coded as 0 2
-      Mf <- Z2 %*% M2 - 1;  # Z.dent %*% M.dent - 1   # convert to -1,1
-    }
-    
-    ## marker matrix coded as additive -1,0,1
-    Mdf <- (Md + Mf)*(1/2) # normal marker matrix for the hybrids
-    rownames(Mdf) <- hyb.names
-    #hist(Mdf)
-    
-    ## dominance matrix for hybrids (0,1 coded)
-    Delta <- 1/2*(1 - Md * Mf) #performs element wise multiplication = Hadamard product
-    rownames(Delta) <- hyb.names
     #hist(Delta)
-    message("Done!!\n")
-    return(list(HMM.add=Mdf, HMM.dom=Delta, data.used=pheno))
+    if(verbose){cat("Done!!\n")}
+    return(list(HMM.add=HMM.add, HMM.dom=HMM.dom, data.used=pheno))
     
   }else{
     return(list(HMM.add=NA, HMM.dom=NA, data.used=pheno))
